@@ -35,11 +35,44 @@ static char current_walfile_name[MAXPGPATH] = "";
 static PGresult *HandleCopyStream(PGconn *conn, XLogRecPtr startpos,
 				 uint32 timeline, char *basedir,
 			   stream_stop_callback stream_stop, int standby_message_timeout,
-				 char *partial_suffix, XLogRecPtr *stoppos);
+				 char *partial_suffix, XLogRecPtr *stoppos,
+				 bool mark_done);
 
 static bool ReadEndOfStreamingResult(PGresult *res, XLogRecPtr *startpos,
 						 uint32 *timeline);
 
+static bool
+mark_file_as_archived(const char *basedir, const char *fname)
+{
+	int fd;
+	static char tmppath[MAXPGPATH];
+
+	snprintf(tmppath, sizeof(tmppath), "%s/archive_status/%s.done",
+			 basedir, fname);
+
+	fd = open(tmppath, O_WRONLY | O_CREAT | PG_BINARY, S_IRUSR | S_IWUSR);
+	if (fd < 0)
+	{
+		fprintf(stderr, _("%s: could not create archive status file \"%s\": %s\n"),
+				progname, tmppath, strerror(errno));
+		return false;
+	}
+
+	if (fsync(fd) != 0)
+	{
+		fprintf(stderr, _("%s: could not fsync file \"%s\": %s\n"),
+				progname, tmppath, strerror(errno));
+
+		close(fd);
+
+		return false;
+	}
+
+	close(fd);
+
+	return true;
+}
+
 /*
  * Open a new WAL file in the specified directory.
  *
@@ -133,7 +166,7 @@ open_walfile(XLogRecPtr startpoint, uint32 timeline, char *basedir,
  * and returns false, otherwise returns true.
  */
 static bool
-close_walfile(char *basedir, char *partial_suffix)
+close_walfile(char *basedir, char *partial_suffix, bool mark_done)
 {
 	off_t		currpos;
 
@@ -166,8 +199,7 @@ close_walfile(char *basedir, char *partial_suffix)
 	walfile = -1;
 
 	/*
-	 * Rename the .partial file only if we've completed writing the whole
-	 * segment or segment_complete is true.
+	 * If we finished writing a .partial file, rename it into place.
 	 */
 	if (currpos == XLOG_SEG_SIZE && partial_suffix)
 	{
@@ -188,6 +220,19 @@ close_walfile(char *basedir, char *partial_suffix)
 				_("%s: not renaming \"%s%s\", segment is not complete\n"),
 				progname, current_walfile_name, partial_suffix);
 
+	/*
+	 * Mark file as archived if requested by the caller - pg_basebackup needs
+	 * to do so as files can otherwise get archived again after promotion of a
+	 * new node. This is in line with walreceiver.c always doing a
+	 * XLogArchiveForceDone() after a complete segment.
+	 */
+	if (currpos == XLOG_SEG_SIZE && mark_done)
+	{
+		/* writes error message if failed */
+		if (!mark_file_as_archived(basedir, current_walfile_name))
+			return false;
+	}
+
 	return true;
 }
 
@@ -286,7 +331,8 @@ existsTimeLineHistoryFile(char *basedir, TimeLineID tli)
 }
 
 static bool
-writeTimeLineHistoryFile(char *basedir, TimeLineID tli, char *filename, char *content)
+writeTimeLineHistoryFile(char *basedir, TimeLineID tli, char *filename,
+						 char *content, bool mark_done)
 {
 	int			size = strlen(content);
 	char		path[MAXPGPATH];
@@ -306,6 +352,8 @@ writeTimeLineHistoryFile(char *basedir, TimeLineID tli, char *filename, char *co
 		return false;
 	}
 
+	snprintf(path, sizeof(path), "%s/%s", basedir, histfname);
+
 	/*
 	 * Write into a temp file name.
 	 */
@@ -356,8 +404,6 @@ writeTimeLineHistoryFile(char *basedir, TimeLineID tli, char *filename, char *co
 	/*
 	 * Now move the completed history file into place with its final name.
 	 */
-
-	snprintf(path, sizeof(path), "%s/%s", basedir, histfname);
 	if (rename(tmppath, path) < 0)
 	{
 		fprintf(stderr, _("%s: could not rename file \"%s\" to \"%s\": %s\n"),
@@ -365,6 +411,14 @@ writeTimeLineHistoryFile(char *basedir, TimeLineID tli, char *filename, char *co
 		return false;
 	}
 
+	/* Maintain archive_status, check close_walfile() for details. */
+	if (mark_done)
+	{
+		/* writes error message if failed */
+		if (!mark_file_as_archived(basedir, histfname))
+			return false;
+	}
+
 	return true;
 }
 
@@ -509,7 +563,8 @@ bool
 ReceiveXlogStream(PGconn *conn, XLogRecPtr startpos, uint32 timeline,
 				  char *sysidentifier, char *basedir,
 				  stream_stop_callback stream_stop,
-				  int standby_message_timeout, char *partial_suffix)
+				  int standby_message_timeout, char *partial_suffix,
+				  bool mark_done)
 {
 	char		query[128];
 	PGresult   *res;
@@ -594,7 +649,8 @@ ReceiveXlogStream(PGconn *conn, XLogRecPtr startpos, uint32 timeline,
 			/* Write the history file to disk */
 			writeTimeLineHistoryFile(basedir, timeline,
 									 PQgetvalue(res, 0, 0),
-									 PQgetvalue(res, 0, 1));
+									 PQgetvalue(res, 0, 1),
+									 mark_done);
 
 			PQclear(res);
 		}
@@ -623,7 +679,7 @@ ReceiveXlogStream(PGconn *conn, XLogRecPtr startpos, uint32 timeline,
 		/* Stream the WAL */
 		res = HandleCopyStream(conn, startpos, timeline, basedir, stream_stop,
 							   standby_message_timeout, partial_suffix,
-							   &stoppos);
+							   &stoppos, mark_done);
 		if (res == NULL)
 			goto error;
 
@@ -681,6 +737,7 @@ ReceiveXlogStream(PGconn *conn, XLogRecPtr startpos, uint32 timeline,
 				fprintf(stderr,
 				   _("%s: unexpected termination of replication stream: %s"),
 						progname, PQresultErrorMessage(res));
+				PQclear(res);
 				goto error;
 			}
 			PQclear(res);
@@ -695,6 +752,8 @@ ReceiveXlogStream(PGconn *conn, XLogRecPtr startpos, uint32 timeline,
 		}
 		else if (PQresultStatus(res) == PGRES_COMMAND_OK)
 		{
+			PQclear(res);
+
 			/*
 			 * End of replication (ie. controlled shut down of the server).
 			 *
@@ -716,6 +775,7 @@ ReceiveXlogStream(PGconn *conn, XLogRecPtr startpos, uint32 timeline,
 			fprintf(stderr,
 					_("%s: unexpected termination of replication stream: %s"),
 					progname, PQresultErrorMessage(res));
+			PQclear(res);
 			goto error;
 		}
 	}
@@ -784,7 +844,7 @@ static PGresult *
 HandleCopyStream(PGconn *conn, XLogRecPtr startpos, uint32 timeline,
 				 char *basedir, stream_stop_callback stream_stop,
 				 int standby_message_timeout, char *partial_suffix,
-				 XLogRecPtr *stoppos)
+				 XLogRecPtr *stoppos, bool mark_done)
 {
 	char	   *copybuf = NULL;
 	int64		last_status = -1;
@@ -811,7 +871,7 @@ HandleCopyStream(PGconn *conn, XLogRecPtr startpos, uint32 timeline,
 		 */
 		if (still_sending && stream_stop(blockpos, timeline, false))
 		{
-			if (!close_walfile(basedir, partial_suffix))
+			if (!close_walfile(basedir, partial_suffix, mark_done))
 			{
 				/* Potential error message is written by close_walfile */
 				goto error;
@@ -910,9 +970,10 @@ HandleCopyStream(PGconn *conn, XLogRecPtr startpos, uint32 timeline,
 			 */
 			if (still_sending)
 			{
-				if (!close_walfile(basedir, partial_suffix))
+				if (!close_walfile(basedir, partial_suffix, mark_done))
 				{
 					/* Error message written in close_walfile() */
+					PQclear(res);
 					goto error;
 				}
 				if (PQresultStatus(res) == PGRES_COPY_IN)
@@ -922,8 +983,10 @@ HandleCopyStream(PGconn *conn, XLogRecPtr startpos, uint32 timeline,
 						fprintf(stderr,
 								_("%s: could not send copy-end packet: %s"),
 								progname, PQerrorMessage(conn));
+						PQclear(res);
 						goto error;
 					}
+					PQclear(res);
 					res = PQgetResult(conn);
 				}
 				still_sending = false;
@@ -990,7 +1053,7 @@ HandleCopyStream(PGconn *conn, XLogRecPtr startpos, uint32 timeline,
 			hdr_len += 8;		/* dataStart */
 			hdr_len += 8;		/* walEnd */
 			hdr_len += 8;		/* sendTime */
-			if (r < hdr_len + 1)
+			if (r < hdr_len)
 			{
 				fprintf(stderr, _("%s: streaming header too small: %d\n"),
 						progname, r);
@@ -1075,13 +1138,13 @@ HandleCopyStream(PGconn *conn, XLogRecPtr startpos, uint32 timeline,
 				/* Did we reach the end of a WAL segment? */
 				if (blockpos % XLOG_SEG_SIZE == 0)
 				{
-					if (!close_walfile(basedir, partial_suffix))
+					if (!close_walfile(basedir, partial_suffix, mark_done))
 						/* Error message written in close_walfile() */
 						goto error;
 
 					xlogoff = 0;
 
-					if (still_sending && stream_stop(blockpos, timeline, false))
+					if (still_sending && stream_stop(blockpos, timeline, true))
 					{
 						if (PQputCopyEnd(conn, NULL) <= 0 || PQflush(conn))
 						{
