@@ -2,7 +2,7 @@
  *
  * reindexdb
  *
- * Portions Copyright (c) 1996-2011, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2013, PostgreSQL Global Development Group
  *
  * src/bin/scripts/reindexdb.c
  *
@@ -19,7 +19,8 @@ static void reindex_one_database(const char *name, const char *dbname,
 					 const char *port, const char *username,
 					 enum trivalue prompt_password, const char *progname,
 					 bool echo);
-static void reindex_all_databases(const char *host, const char *port,
+static void reindex_all_databases(const char *maintenance_db,
+					  const char *host, const char *port,
 					  const char *username, enum trivalue prompt_password,
 					  const char *progname, bool echo,
 					  bool quiet);
@@ -45,6 +46,7 @@ main(int argc, char *argv[])
 		{"system", no_argument, NULL, 's'},
 		{"table", required_argument, NULL, 't'},
 		{"index", required_argument, NULL, 'i'},
+		{"maintenance-db", required_argument, NULL, 2},
 		{NULL, 0, NULL, 0}
 	};
 
@@ -53,6 +55,7 @@ main(int argc, char *argv[])
 	int			c;
 
 	const char *dbname = NULL;
+	const char *maintenance_db = NULL;
 	const char *host = NULL;
 	const char *port = NULL;
 	const char *username = NULL;
@@ -61,8 +64,8 @@ main(int argc, char *argv[])
 	bool		alldb = false;
 	bool		echo = false;
 	bool		quiet = false;
-	const char *table = NULL;
-	const char *index = NULL;
+	SimpleStringList indexes = {NULL, NULL};
+	SimpleStringList tables = {NULL, NULL};
 
 	progname = get_progname(argv[0]);
 	set_pglocale_pgservice(argv[0], PG_TEXTDOMAIN("pgscripts"));
@@ -75,13 +78,13 @@ main(int argc, char *argv[])
 		switch (c)
 		{
 			case 'h':
-				host = optarg;
+				host = pg_strdup(optarg);
 				break;
 			case 'p':
-				port = optarg;
+				port = pg_strdup(optarg);
 				break;
 			case 'U':
-				username = optarg;
+				username = pg_strdup(optarg);
 				break;
 			case 'w':
 				prompt_password = TRI_NO;
@@ -96,7 +99,7 @@ main(int argc, char *argv[])
 				quiet = true;
 				break;
 			case 'd':
-				dbname = optarg;
+				dbname = pg_strdup(optarg);
 				break;
 			case 'a':
 				alldb = true;
@@ -105,10 +108,13 @@ main(int argc, char *argv[])
 				syscatalog = true;
 				break;
 			case 't':
-				table = optarg;
+				simple_string_list_append(&tables, optarg);
 				break;
 			case 'i':
-				index = optarg;
+				simple_string_list_append(&indexes, optarg);
+				break;
+			case 2:
+				maintenance_db = pg_strdup(optarg);
 				break;
 			default:
 				fprintf(stderr, _("Try \"%s --help\" for more information.\n"), progname);
@@ -116,17 +122,22 @@ main(int argc, char *argv[])
 		}
 	}
 
-	switch (argc - optind)
+	/*
+	 * Non-option argument specifies database name as long as it wasn't
+	 * already specified with -d / --dbname
+	 */
+	if (optind < argc && dbname == NULL)
 	{
-		case 0:
-			break;
-		case 1:
-			dbname = argv[optind];
-			break;
-		default:
-			fprintf(stderr, _("%s: too many command-line arguments (first is \"%s\")\n"), progname, argv[optind + 1]);
-			fprintf(stderr, _("Try \"%s --help\" for more information.\n"), progname);
-			exit(1);
+		dbname = argv[optind];
+		optind++;
+	}
+
+	if (optind < argc)
+	{
+		fprintf(stderr, _("%s: too many command-line arguments (first is \"%s\")\n"),
+				progname, argv[optind]);
+		fprintf(stderr, _("Try \"%s --help\" for more information.\n"), progname);
+		exit(1);
 	}
 
 	setup_cancel_handler();
@@ -143,30 +154,30 @@ main(int argc, char *argv[])
 			fprintf(stderr, _("%s: cannot reindex all databases and system catalogs at the same time\n"), progname);
 			exit(1);
 		}
-		if (table)
+		if (tables.head != NULL)
 		{
-			fprintf(stderr, _("%s: cannot reindex a specific table in all databases\n"), progname);
+			fprintf(stderr, _("%s: cannot reindex specific table(s) in all databases\n"), progname);
 			exit(1);
 		}
-		if (index)
+		if (indexes.head != NULL)
 		{
-			fprintf(stderr, _("%s: cannot reindex a specific index in all databases\n"), progname);
+			fprintf(stderr, _("%s: cannot reindex specific index(es) in all databases\n"), progname);
 			exit(1);
 		}
 
-		reindex_all_databases(host, port, username, prompt_password,
-							  progname, echo, quiet);
+		reindex_all_databases(maintenance_db, host, port, username,
+							  prompt_password, progname, echo, quiet);
 	}
 	else if (syscatalog)
 	{
-		if (table)
+		if (tables.head != NULL)
 		{
-			fprintf(stderr, _("%s: cannot reindex a specific table and system catalogs at the same time\n"), progname);
+			fprintf(stderr, _("%s: cannot reindex specific table(s) and system catalogs at the same time\n"), progname);
 			exit(1);
 		}
-		if (index)
+		if (indexes.head != NULL)
 		{
-			fprintf(stderr, _("%s: cannot reindex a specific index and system catalogs at the same time\n"), progname);
+			fprintf(stderr, _("%s: cannot reindex specific index(es) and system catalogs at the same time\n"), progname);
 			exit(1);
 		}
 
@@ -195,15 +206,29 @@ main(int argc, char *argv[])
 				dbname = get_user_name(progname);
 		}
 
-		if (index)
-			reindex_one_database(index, dbname, "INDEX", host, port,
-								 username, prompt_password, progname, echo);
-		if (table)
-			reindex_one_database(table, dbname, "TABLE", host, port,
-								 username, prompt_password, progname, echo);
-		/* reindex database only if index or table is not specified */
-		if (index == NULL && table == NULL)
-			reindex_one_database(NULL, dbname, "DATABASE", host, port,
+		if (indexes.head != NULL)
+		{
+			SimpleStringListCell *cell;
+
+			for (cell = indexes.head; cell; cell = cell->next)
+			{
+				reindex_one_database(cell->val, dbname, "INDEX", host, port,
+								  username, prompt_password, progname, echo);
+			}
+		}
+		if (tables.head != NULL)
+		{
+			SimpleStringListCell *cell;
+
+			for (cell = tables.head; cell; cell = cell->next)
+			{
+				reindex_one_database(cell->val, dbname, "TABLE", host, port,
+								  username, prompt_password, progname, echo);
+			}
+		}
+		/* reindex database only if neither index nor table is specified */
+		if (indexes.head == NULL && tables.head == NULL)
+			reindex_one_database(dbname, dbname, "DATABASE", host, port,
 								 username, prompt_password, progname, echo);
 	}
 
@@ -219,30 +244,31 @@ reindex_one_database(const char *name, const char *dbname, const char *type,
 
 	PGconn	   *conn;
 
-	conn = connectDatabase(dbname, host, port, username, prompt_password, progname);
-
 	initPQExpBuffer(&sql);
 
 	appendPQExpBuffer(&sql, "REINDEX");
 	if (strcmp(type, "TABLE") == 0)
-		appendPQExpBuffer(&sql, " TABLE %s", fmtId(name));
+		appendPQExpBuffer(&sql, " TABLE %s", name);
 	else if (strcmp(type, "INDEX") == 0)
-		appendPQExpBuffer(&sql, " INDEX %s", fmtId(name));
+		appendPQExpBuffer(&sql, " INDEX %s", name);
 	else if (strcmp(type, "DATABASE") == 0)
-		appendPQExpBuffer(&sql, " DATABASE %s", fmtId(PQdb(conn)));
+		appendPQExpBuffer(&sql, " DATABASE %s", fmtId(name));
 	appendPQExpBuffer(&sql, ";\n");
 
+	conn = connectDatabase(dbname, host, port, username, prompt_password,
+						   progname, false);
+
 	if (!executeMaintenanceCommand(conn, sql.data, echo))
 	{
 		if (strcmp(type, "TABLE") == 0)
 			fprintf(stderr, _("%s: reindexing of table \"%s\" in database \"%s\" failed: %s"),
-					progname, name, PQdb(conn), PQerrorMessage(conn));
+					progname, name, dbname, PQerrorMessage(conn));
 		if (strcmp(type, "INDEX") == 0)
 			fprintf(stderr, _("%s: reindexing of index \"%s\" in database \"%s\" failed: %s"),
-					progname, name, PQdb(conn), PQerrorMessage(conn));
+					progname, name, dbname, PQerrorMessage(conn));
 		else
 			fprintf(stderr, _("%s: reindexing of database \"%s\" failed: %s"),
-					progname, PQdb(conn), PQerrorMessage(conn));
+					progname, dbname, PQerrorMessage(conn));
 		PQfinish(conn);
 		exit(1);
 	}
@@ -252,20 +278,20 @@ reindex_one_database(const char *name, const char *dbname, const char *type,
 }
 
 static void
-reindex_all_databases(const char *host, const char *port,
+reindex_all_databases(const char *maintenance_db,
+					  const char *host, const char *port,
 					  const char *username, enum trivalue prompt_password,
 					  const char *progname, bool echo, bool quiet)
 {
 	PGconn	   *conn;
 	PGresult   *result;
-	PQExpBufferData connstr;
 	int			i;
 
-	conn = connectDatabase("postgres", host, port, username, prompt_password, progname);
+	conn = connectMaintenanceDatabase(maintenance_db, host, port, username,
+									  prompt_password, progname);
 	result = executeQuery(conn, "SELECT datname FROM pg_database WHERE datallowconn ORDER BY 1;", progname, echo);
 	PQfinish(conn);
 
-	initPQExpBuffer(&connstr);
 	for (i = 0; i < PQntuples(result); i++)
 	{
 		char	   *dbname = PQgetvalue(result, i, 0);
@@ -276,15 +302,9 @@ reindex_all_databases(const char *host, const char *port,
 			fflush(stdout);
 		}
 
-		resetPQExpBuffer(&connstr);
-		appendPQExpBuffer(&connstr, "dbname=");
-		appendConnStrVal(&connstr, dbname);
-
-		reindex_one_database(NULL, connstr.data, "DATABASE", host,
-							 port, username, prompt_password,
-							 progname, echo);
+		reindex_one_database(dbname, dbname, "DATABASE", host, port, username,
+							 prompt_password, progname, echo);
 	}
-	termPQExpBuffer(&connstr);
 
 	PQclear(result);
 }
@@ -294,15 +314,16 @@ reindex_system_catalogs(const char *dbname, const char *host, const char *port,
 						const char *username, enum trivalue prompt_password,
 						const char *progname, bool echo)
 {
-	PGconn	   *conn;
 	PQExpBufferData sql;
 
-	conn = connectDatabase(dbname, host, port, username, prompt_password, progname);
+	PGconn	   *conn;
 
 	initPQExpBuffer(&sql);
 
-	appendPQExpBuffer(&sql, "REINDEX SYSTEM %s;\n", fmtId(PQdb(conn)));
+	appendPQExpBuffer(&sql, "REINDEX SYSTEM %s;\n", dbname);
 
+	conn = connectDatabase(dbname, host, port, username, prompt_password,
+						   progname, false);
 	if (!executeMaintenanceCommand(conn, sql.data, echo))
 	{
 		fprintf(stderr, _("%s: reindexing of system catalogs failed: %s"),
@@ -324,18 +345,19 @@ help(const char *progname)
 	printf(_("  -a, --all                 reindex all databases\n"));
 	printf(_("  -d, --dbname=DBNAME       database to reindex\n"));
 	printf(_("  -e, --echo                show the commands being sent to the server\n"));
-	printf(_("  -i, --index=INDEX         recreate specific index only\n"));
+	printf(_("  -i, --index=INDEX         recreate specific index(es) only\n"));
 	printf(_("  -q, --quiet               don't write any messages\n"));
 	printf(_("  -s, --system              reindex system catalogs\n"));
-	printf(_("  -t, --table=TABLE         reindex specific table only\n"));
-	printf(_("  --help                    show this help, then exit\n"));
-	printf(_("  --version                 output version information, then exit\n"));
+	printf(_("  -t, --table=TABLE         reindex specific table(s) only\n"));
+	printf(_("  -V, --version             output version information, then exit\n"));
+	printf(_("  -?, --help                show this help, then exit\n"));
 	printf(_("\nConnection options:\n"));
 	printf(_("  -h, --host=HOSTNAME       database server host or socket directory\n"));
 	printf(_("  -p, --port=PORT           database server port\n"));
 	printf(_("  -U, --username=USERNAME   user name to connect as\n"));
 	printf(_("  -w, --no-password         never prompt for password\n"));
 	printf(_("  -W, --password            force password prompt\n"));
+	printf(_("  --maintenance-db=DBNAME   alternate maintenance database\n"));
 	printf(_("\nRead the description of the SQL command REINDEX for details.\n"));
 	printf(_("\nReport bugs to <pgsql-bugs@postgresql.org>.\n"));
 }
